@@ -23,7 +23,7 @@ class IMAPSequenceSetTest < Test::Unit::TestCase
     assert seqset.cover? sorted.sample 100
   end
 
-  test "compared to reference Set, add many random values" do
+  test "fuzz test: add numbers and compare to reference Set" do
     set    = Set.new
     seqset = SequenceSet.new
     10.times do
@@ -32,7 +32,7 @@ class IMAPSequenceSetTest < Test::Unit::TestCase
     end
   end
 
-  test "compared to reference Set, add many large ranges" do
+  test "fuzz test: add ranges and compare to reference Set" do
     set    = Set.new
     seqset = SequenceSet.new
     (1..10_000).each_slice(250) do
@@ -41,6 +41,80 @@ class IMAPSequenceSetTest < Test::Unit::TestCase
     end
   end
 
+  test "fuzz test: set union identities" do
+    10.times do
+      lhs = SequenceSet[Array.new(100) { rand(1..300) }]
+      rhs = SequenceSet[Array.new(100) { rand(1..300) }]
+      union = lhs | rhs
+      assert_equal union, rhs | lhs # commutative
+      assert_equal union, ~(~lhs & ~rhs) # De Morgan's Law
+      assert_equal union, lhs | (lhs ^ rhs)
+      assert_equal union, lhs | (rhs - lhs)
+      assert_equal union, (lhs & rhs) ^ (lhs ^ rhs)
+      mutable = lhs.dup
+      assert_equal union, mutable.merge(rhs)
+      assert_equal union, mutable
+    end
+  end
+
+  test "fuzz test: set intersection identities" do
+    10.times do
+      lhs = SequenceSet[Array.new(100) { rand(1..300) }]
+      rhs = SequenceSet[Array.new(100) { rand(1..300) }]
+      intersection = lhs & rhs
+      assert_equal intersection, rhs & lhs # commutative
+      assert_equal intersection, ~(~lhs | ~rhs) # De Morgan's Law
+      assert_equal intersection, lhs - ~rhs
+      assert_equal intersection, lhs - (lhs - rhs)
+      assert_equal intersection, lhs - (lhs ^ rhs)
+      assert_equal intersection, lhs ^ (lhs - rhs)
+    end
+  end
+
+  test "fuzz test: set subtraction identities" do
+    10.times do
+      lhs = SequenceSet[Array.new(100) { rand(1..300) }]
+      rhs = SequenceSet[Array.new(100) { rand(1..300) }]
+      difference = lhs - rhs
+      assert_equal difference, ~rhs - ~lhs
+      assert_equal difference, ~(~lhs | rhs)
+      assert_equal difference, lhs & (lhs ^ rhs)
+      assert_equal difference, lhs ^ (lhs & rhs)
+      assert_equal difference, rhs ^ (lhs | rhs)
+      mutable = lhs.dup
+      assert_equal difference, mutable.subtract(rhs)
+      assert_equal difference, mutable
+    end
+  end
+
+  test "fuzz test: set xor identities" do
+    10.times do
+      lhs = SequenceSet[Array.new(100) { rand(1..300) }]
+      rhs = SequenceSet[Array.new(100) { rand(1..300) }]
+      mid = SequenceSet[Array.new(100) { rand(1..300) }]
+      xor = lhs ^ rhs
+      assert_equal xor, rhs ^ lhs # commutative
+      assert_equal xor, (lhs | rhs) - (lhs & rhs)
+      assert_equal xor, (lhs ^ mid) ^ (mid ^ rhs)
+      assert_equal xor, ~lhs ^ ~rhs
+    end
+  end
+
+  test "fuzz test: set complement identities" do
+    10.times do
+      set = SequenceSet[Array.new(100) { rand(1..300) }]
+      complement = ~set
+      assert_equal set,        ~complement
+      assert_equal complement, ~set.dup
+      assert_equal complement, SequenceSet.full - set
+      mutable = set.dup
+      assert_equal complement, mutable.complement!
+      assert_equal complement, mutable
+      assert_equal set,        mutable.complement!
+      assert_equal set,        mutable
+    end
+  end
+
   test "#== equality by value (not by identity or representation)" do
     assert_equal SequenceSet.new, SequenceSet.new
     assert_equal SequenceSet.new("1"), SequenceSet[1]
@@ -83,6 +157,34 @@ class IMAPSequenceSetTest < Test::Unit::TestCase
     end
   end
 
+  data "#slice(length)",   {transform: ->{ _1.slice(0, 10)    }, }
+  data "#slice(range)",    {transform: ->{ _1.slice(0...10)   }, }
+  data "#slice => empty",  {transform: ->{ _1.slice(0...0)    }, }
+  data "#slice => empty",  {transform: ->{ _1.slice(10..9)    }, }
+  data "#union",           {transform: ->{ _1 | (1..100)      }, }
+  data "#intersection",    {transform: ->{ _1 & (1..100)      }, }
+  data "#difference",      {transform: ->{ _1 - (1..100)      }, }
+  data "#xor",             {transform: ->{ _1 ^ (1..100)      }, }
+  data "#complement",      {transform: ->{ ~_1                }, }
+  data "#normalize",       {transform: ->{ _1.normalize       }, }
+  data "#limit",           {transform: ->{ _1.limit(max: 22)  }, freeze: :always }
+  data "#limit => empty",  {transform: ->{ _1.limit(max: 1)   }, freeze: :always }
+  test "transforms keep frozen status" do |data|
+    transform = data.fetch(:transform)
+    set = SequenceSet.new("2:4,7:11,99,999")
+    dup = set.dup
+    result = transform.to_proc.(set)
+    assert_equal dup, set, "transform should not modified"
+    if data[:freeze] == :always
+      assert result.frozen?, "this transform always returns frozen"
+    else
+      refute result.frozen?, "transform of non-frozen returned frozen"
+    end
+    set.freeze
+    result = transform.to_proc.(set)
+    assert result.frozen?, "transform of frozen returned non-frozen"
+  end
+
   %i[clone dup].each do |method|
     test "##{method}" do
       orig = SequenceSet.new "2:4,7:11,99,999"
@@ -265,6 +367,9 @@ class IMAPSequenceSetTest < Test::Unit::TestCase
                  SequenceSet[((1..10_000) % 10).to_a][-5, 4]
     assert_nil SequenceSet[111..222, 888..999][2000, 4]
     assert_nil SequenceSet[111..222, 888..999][-2000, 4]
+    # with length longer than the remaining members
+    assert_equal SequenceSet[101...200],
+                 SequenceSet[1...200][100, 10000]
   end
 
   test "#[range]" do
@@ -286,9 +391,13 @@ class IMAPSequenceSetTest < Test::Unit::TestCase
     assert_equal SequenceSet.empty, SequenceSet[1..100][-50..-60]
     assert_equal SequenceSet.empty, SequenceSet[1..100][-10..10]
     assert_equal SequenceSet.empty, SequenceSet[1..100][60..-60]
+    assert_equal SequenceSet.empty, SequenceSet[1..100][10...0]
+    assert_equal SequenceSet.empty, SequenceSet[1..100][0...0]
     assert_nil SequenceSet.empty[2..4]
     assert_nil SequenceSet[101..200][1000..1060]
     assert_nil SequenceSet[101..200][-1000..-60]
+    # with length longer than the remaining members
+    assert_equal SequenceSet[101..1111], SequenceSet[1..1111][100..999_999]
   end
 
   test "#find_index" do
@@ -538,6 +647,16 @@ class IMAPSequenceSetTest < Test::Unit::TestCase
     assert_equal seqset["1,5,11:99"], seqset["1,5:6,8:9,11:99"].subtract("6:9")
   end
 
+  test "#xor" do
+    seqset = -> { SequenceSet.new(_1) }
+    assert_equal seqset["1:5,11:15"], seqset["1:10"] ^ seqset["6:15"]
+    assert_equal seqset["1,3,5:6"],   seqset[1..5]   ^ [2, 4, 6]
+    assert_equal SequenceSet.empty,   seqset[1..5]   ^ seqset[1..5]
+    assert_equal seqset["1:100"],     seqset["1:50"] ^ seqset["51:100"]
+    assert_equal seqset["1:50"],      seqset["1:50"] ^ SequenceSet.empty
+    assert_equal seqset["1:50"],      SequenceSet.empty ^ seqset["1:50"]
+  end
+
   test "#min" do
     assert_equal   3, SequenceSet.new("34:3").min
     assert_equal 345, SequenceSet.new("345,678").min
